!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
module image_def
  use gildas_def
  use gbl_format
  !
  integer, parameter :: code_read_data     =-1
  integer, parameter :: code_read_header   =-2
  integer, parameter :: code_update_header =-3
  integer, parameter :: code_write_header  =-4
  integer, parameter :: code_write_data    =-5
  integer, parameter :: code_read_image    =-6
  integer, parameter :: code_free_image    =-7
  integer, parameter :: code_create_image  =-8
  !
  ! /XIPAR/
  type :: location
    sequence
    integer(kind=4) :: al64 = 0               ! Padding to align ADDR
    integer(kind=4) :: size = 0               ! Size of image
    integer(kind=address_length) :: addr = 0  ! Address of image
    integer :: islo         = 0               ! Image Slot number
    integer :: mslo         = 0               ! Memory Slot number
    logical :: read         = .true.          ! ReadOnly status
    logical :: getvm        = .false.         ! Memory / File indicator
  end type location
  !
  ! /XCPAR/
  type :: strings
    sequence
    character(len=12) :: type    = 'GILDAS_IMAGE'  !     Image Type (see ITYP)
    character(len=12) :: unit    = ' '             !     Data Units (see IUNI)
    character(len=12) :: code(4) = ' '             ! 59  Axis codes (see ICOD)
    character(len=12) :: syst    = ' '             ! 71  Coordinate system (see ISYS)
    character(len=12) :: name    = ' '             ! 75  Source name (see ISOU)
    character(len=12) :: line    = ' '             !     Line name (see ILIN)
  end type strings
  !
  ! /XPAR/
  type :: gildas_header
    sequence
    integer(kind=4) :: ityp(3) = 0        !  1  Image Type
    integer(kind=4) :: form    = fmt_r4   !  4  Data format (FMT_xx)
    integer(kind=4) :: nvb     = 0        !  5  Number of blocks
    integer(kind=4) :: fill(5) = 0        !  6  Reserved space
    integer(kind=4) :: gene    = 29       ! 11  General section length
    integer(kind=4) :: ndim    = 0        ! 12  Number of dimensions
    integer(kind=4) :: dim(4)  = 0        ! 13  Dimensions
    real(kind=8) :: ref1 = 1.0            ! 17  Reference pixel
    real(kind=8) :: val1 = 1.0            ! 19  Value at reference pixel
    real(kind=8) :: inc1 = 1.0            ! 21  Increment per pixel
    real(kind=8) :: ref2 = 1.0            ! 23  idem for axis 2
    real(kind=8) :: val2 = 1.0            ! 25
    real(kind=8) :: inc2 = 1.0            ! 27
    real(kind=8) :: ref3 = 1.0            ! 29  idem for axis 3
    real(kind=8) :: val3 = 1.0            ! 31
    real(kind=8) :: inc3 = 1.0            ! 33
    real(kind=8) :: ref4 = 1.0            ! 35  idem for axis 4
    real(kind=8) :: val4 = 1.0            ! 37
    real(kind=8) :: inc4 = 1.0            ! 39
    !
    integer(kind=4) :: blan = 2           ! 41  Blanking section length
    real(kind=4) :: bval = +1.23456e+34   ! 42  Blanking value
    real(kind=4) :: eval = -1.0           ! 43  Tolerance
    !
    integer(kind=4) :: extr = 0           ! 44  Extrema section length
    real(kind=4) :: rmin    = 0.0         ! 45  Minimum
    real(kind=4) :: rmax    = 0.0         ! 46  Maximum
    integer(kind=4) :: min1 = 0           ! 47  Pixel of min on axis 1
    integer(kind=4) :: max1 = 0           ! 48  Pixel of max ......
    integer(kind=4) :: min2 = 0           ! 49  idem for axis 2
    integer(kind=4) :: max2 = 0           ! 50
    integer(kind=4) :: min3 = 0           ! 51           axis 3
    integer(kind=4) :: max3 = 0           ! 52
    integer(kind=4) :: min4 = 0           ! 53           axis 4
    integer(kind=4) :: max4 = 0           ! 54
    !
    integer(kind=4) :: desc      = 18     ! 55  Description section length
    integer(kind=4) :: iuni(3)   = 0      ! 56  Data Unit
    integer(kind=4) :: icod(3,4) = 0      ! 59  Axis names
    integer(kind=4) :: isys(3)   = 0      ! 71  Coordinate System
    integer(kind=4) :: dum1      = 0      ! Void for padding
    !
    integer(kind=4) :: posi    = 12       ! 74  Position section length
    integer(kind=4) :: isou(3) = 0        ! 75  Source name
    real(kind=8) :: ra         = 0.0      ! 78  Right Ascension
    real(kind=8) :: dec        = 0.0      ! 80  Declination
    real(kind=8) :: lii        = 0.0      ! 82  Galactic longitude
    real(kind=8) :: bii        = 0.0      ! 84           latitude
    real(kind=4) :: epoc       = 0.0      ! 86  Epoch of coordinates
    integer(kind=4) :: dum2    = 0        ! Void for padding
    !
    integer(kind=4) :: proj = 9           ! 87  Projection section length
    integer(kind=4) :: ptyp = 0           ! 88  Projection type
    real(kind=8) :: a0      = 0.0         ! 89  X of projection center
    real(kind=8) :: d0      = 0.0         ! 91  Y of projection center
    real(kind=8) :: pang    = 0.0         ! 93  Projection angle
    integer(kind=4) :: xaxi = 0           ! 95  X axis
    integer(kind=4) :: yaxi = 0           ! 96  Y axis
    !
    integer(kind=4) :: spec    = 12       ! 97  Spectroscopy section length
    integer(kind=4) :: ilin(3) = 0        ! 98  Line name
    real(kind=8) :: fres       = 0.0      !101  Frequency resolution
    real(kind=8) :: fima       = 0.0      !103  Image frequency
    real(kind=8) :: freq       = 0.0      !105  Rest Frequency
    real(kind=4) :: vres       = 0.0      !107  Velocity resolution
    real(kind=4) :: voff       = 0.0      !108  Velocity offset
    integer(kind=4) :: faxi    = 0        !109  Frequency axis
    !
    integer(kind=4) :: reso = 0           !110  Resolution section length
    real(kind=4) :: majo    = 0.0         !111  Major axis
    real(kind=4) :: mino    = 0.0         !112  Minor axis
    real(kind=4) :: posa    = 0.0         !113  Position angle
    !
    integer(kind=4) :: sigm = 0           ! 114 Noise section length
    real(kind=4) :: noise   = 0.0         ! 115 Theoretical noise
    real(kind=4) :: rms     = 0.0         ! 116 Actual noise
    !
    integer(kind=4) :: prop  = 0          ! 117 Proper motion section length
    real(kind=4) :: mura     = 0.0        ! 118 along RA, in mas/yr
    real(kind=4) :: mudec    = 0.0        ! 119 along Dec, in mas/yr
    real(kind=4) :: parallax = 0.0        ! 120 in mas
  ! real(kind=4) :: pepoch   = 2000.0     ! 121 in yrs ?
  end type gildas_header
  !
  type :: sic_header
    sequence
    integer(kind=4) :: ityp(3)   = 0       !  1
    integer(kind=4) :: form      = fmt_r4  !  4
    integer(kind=4) :: nvb       = 0       !  5
    integer(kind=4) :: fill(5)   = 0       !  6
    integer(kind=4) :: gene      = 29      ! 11
    integer(kind=4) :: ndim      = 0       ! 12
    integer(kind=4) :: dim(4)    = 0       ! 13
    real(kind=8) :: convert(3,4) = 1.0     ! 17-40
    !
    integer(kind=4) :: blan  = 2                        ! 41
    real(kind=4) :: blank(2) = (/ +1.23456e+34,-1.0 /)  ! 42-43
    !
    integer(kind=4) :: extrema    = 0     ! 44
    real(kind=4) :: min           = 0.0   ! 45
    real(kind=4) :: max           = 0.0   ! 46
    integer(kind=4) :: where(2,4) = 0     ! 47-54
    !
    integer(kind=4) :: desc      = 18     ! 55
    integer(kind=4) :: iuni(3)   = 0      ! 56 Unit
    integer(kind=4) :: icod(3,4) = 0      ! 59 Unit1,2,3,4
    integer(kind=4) :: isys(3)   = 0      ! 71 System
    integer(kind=4) :: dum1      = 0      ! Void
    !
    integer(kind=4) :: posi    = 12       ! 74
    integer(kind=4) :: isou(3) = 0        ! 75  Source
    real(kind=8) :: ra         = 0.0      ! 78
    real(kind=8) :: dec        = 0.0      ! 80
    real(kind=8) :: lii        = 0.0      ! 82
    real(kind=8) :: bii        = 0.0      ! 84
    real(kind=4) :: epoc       = 0.0      ! 86
    integer(kind=4) :: dum2    = 0        ! Void
    !
    integer(kind=4) :: proj   = 9         ! 87
    integer(kind=4) :: ptyp   = 0         ! 88
    real(kind=8) :: a0        = 0.0       ! 89
    real(kind=8) :: d0        = 0.0       ! 91
    real(kind=8) :: angle     = 0.0       ! 93
    integer(kind=4) :: x_axis = 0         ! 95
    integer(kind=4) :: y_axis = 0         ! 96
    !
    integer(kind=4) :: spec    = 12       ! 97
    integer(kind=4) :: ilin(3) = 0        ! 98 Line
    real(kind=8) :: freqres    = 0.0      ! 101
    real(kind=8) :: freqoff    = 0.0      ! 103
    real(kind=8) :: restfre    = 0.0      ! 105
    real(kind=4) :: velres     = 0.0      ! 107
    real(kind=4) :: veloff     = 0.0      ! 108
    integer(kind=4) :: f_axis  = 0        ! 109
    !
    integer(kind=4) :: beam    = 0        ! 110
    real(kind=4) :: major      = 0.0      ! 111
    real(kind=4) :: minor      = 0.0      ! 112
    real(kind=4) :: pa         = 0.0      ! 113
    !
    integer(kind=4) :: sigma   = 0        ! 114
    real(kind=4) :: noise      = 0.0      ! 115
    real(kind=4) :: rms        = 0.0      ! 116
    !
    integer(kind=4) :: proper  = 0        ! 117 Proper motion section length
    real(kind=4) :: mura       = 0.0      ! 118 along RA, in mas/yr
    real(kind=4) :: mudec      = 0.0      ! 119 along Dec, in mas/yr
    real(kind=4) :: parallax   = 0.0      ! 120 in mas
  ! real(kind=4) :: pepoch     = 2000.0   ! 121 in yrs ?
  end type sic_header
  !
  ! Gildas X,Y,Z version
  type :: gildas
    sequence
    character(len=256)   :: file = ' '  ! File name
    type (strings)       :: char        !
    type (location)      :: loca        !
    type (gildas_header) :: gil         !
    integer(kind=4) :: blc(4) = 0       ! Bottom left corner
    integer(kind=4) :: trc(4) = 0       ! Top right corner
    integer(kind=4) :: header = 0       ! Defined / Undefined
    integer(kind=4) :: status = 0       ! Last error code
    real,         pointer :: r1d(:)       => null()  ! Pointer to 1D data
    real(kind=8), pointer :: d1d(:)       => null()
    integer,      pointer :: i1d(:)       => null()
    real,         pointer :: r2d(:,:)     => null()  ! Pointer to 2D data
    real(kind=8), pointer :: d2d(:,:)     => null()
    integer,      pointer :: i2d(:,:)     => null()
    real,         pointer :: r3d(:,:,:)   => null()  ! Pointer to 3D data
    real(kind=8), pointer :: d3d(:,:,:)   => null()
    integer,      pointer :: i3d(:,:,:)   => null()
    real,         pointer :: r4d(:,:,:,:) => null()  ! Pointer to 4D data
    real(kind=8), pointer :: d4d(:,:,:,:) => null()
    integer,      pointer :: i4d(:,:,:,:) => null()
  end type gildas
  !
  ! SIC Variable version
  type :: sic
    sequence
    character(len=256) :: file = ' '      ! File name
    type (strings) :: char
    type (location) :: loca
    type (sic_header) :: sic
    integer(kind=4) :: blc(4) = 0
    integer(kind=4) :: trc(4) = 0
    integer(kind=4) :: header = 0
    integer(kind=4) :: status = 0
    real,         pointer :: r1d(:)       => null()
    real(kind=8), pointer :: d1d(:)       => null()
    integer,      pointer :: i1d(:)       => null()
    real,         pointer :: r2d(:,:)     => null()
    real(kind=8), pointer :: d2d(:,:)     => null()
    integer,      pointer :: i2d(:,:)     => null()
    real,         pointer :: r3d(:,:,:)   => null()
    real(kind=8), pointer :: d3d(:,:,:)   => null()
    integer,      pointer :: i3d(:,:,:)   => null()
    real,         pointer :: r4d(:,:,:,:) => null()
    real(kind=8), pointer :: d4d(:,:,:,:) => null()
    integer,      pointer :: i4d(:,:,:,:) => null()
  end type sic
  !
end module image_def
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
