!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! TELCAL interface definitions
!
! Warning:  Only subroutine and functions provided by the library to the
!           outside world should be here.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
module telcal_interfaces
  !
  ! General TELCAL
  !
  interface ! Load TELCAL environment (Language, global variables, etc...)
     subroutine load_telcal
     end subroutine load_telcal
  end interface
  !
  interface ! Interpret and execute TELCAL commands 
     subroutine run_telcal(line,comm,error)
       character(*) line,comm
       logical error
     end subroutine run_telcal
  end interface
  !
  ! FIT related
  !
  interface ! General unidimensional fitting routine
     subroutine fit_1d(dat,fun,verbose)
       use fit_definitions
       type (simple_1d) :: dat
       type (fit_fun)   :: fun
       logical :: verbose
     end subroutine fit_1d
  end interface
  !
  interface ! Initialize the "dat" instance of the simple_1d derived type
     subroutine null_simple_1d(dat)
       use fit_definitions
       type (simple_1d) :: dat
     end subroutine null_simple_1d
  end interface
  !
  interface ! Initialize the "par" instance of the fit_par derived type
     subroutine null_parameter(par)
       use fit_definitions
       type (fit_par) :: par
     end subroutine null_parameter
  end interface
  !
  interface ! Initialize the "fun" instance of the fit_fun derived type
     subroutine null_function(fun)
       use fit_definitions
       type (fit_fun) :: fun
     end subroutine null_function
  end interface
  !
  interface ! Copy the "in" simple_1d derived type into "out"
     subroutine copy_simple_1d(in,out)
       use fit_definitions
       type (simple_1d) :: in,out
     end subroutine copy_simple_1d
  end interface
  !
  interface ! Copy the "in" fit_fun derived type into "out"
     subroutine copy_function(in,out)
       use fit_definitions
       type (fit_fun) :: in,out
     end subroutine copy_function
  end interface
  !
  ! POINTING related
  !
  !
  ! FOCUS related
  !
  interface ! Initialize the focus structure
     subroutine init_focus(ndat)
       use focus_definitions
       integer :: ndat ! Size of the data arrays
     end subroutine init_focus
  end interface
  !
  interface ! Solve the telescope focus through a forwards and backwards 
            ! displacement 
            ! Then plot results
     subroutine solve_focus
       use fit_definitions
       use focus_definitions
     end subroutine solve_focus
  end interface
end module telcal_interfaces
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
