module plot_formula
  integer, parameter :: plot_length=4
  real(kind=plot_length) :: gx1,gx2,gy1,gy2  ! box location
  real(kind=plot_length) :: gvx, gvx1, gvx2  ! in velocity
  real(kind=plot_length) :: gcx, gcx1, gcx2  ! in channels
  real(kind=plot_length) :: gfx, gfx1, gfx2  ! in rest frequency offset
  real(kind=8)           :: gfxo             ! the rest frequency offset
  real(kind=plot_length) :: gix, gix1, gix2  ! in image offset
  real(kind=8)           :: gixo             ! the image frequency offset
  real(kind=plot_length) :: gux, gux1, gux2  ! in current units
  real(kind=plot_length) :: guy, guy1, guy2  ! on y axis
  real(kind=plot_length) :: guz, guz1, guz2  ! on y axis
end module plot_formula
!<FF>
!
!
!
!
module class_popup 
  ! A voir
  integer, allocatable :: ipop(:)
  real,    allocatable :: xpop(:)
  real,    allocatable :: ypop(:)
  integer :: npop,cpop,lpop
  real    :: pgx1,pgx2,pux1,pux2
  real    :: pgy1,pgy2,puy1,puy2
  real    :: pgz1,pgz2,puz1,puz2
end module class_popup
!<FF>
!
!
!
!
module class_common
  use gildas_def
  !
  integer, parameter :: msec=64          ! nombre maximal de sections
  integer, parameter :: mex=251          ! nombre max d'extensions de l'index
  !
  type indexblock
     sequence
     logical update   ! Is file opened for Update ?
     logical single   ! Should ObsNum be unique ?
     integer code     ! code fichier
     integer next     ! prochain bloc libre
     integer ilex     ! longueur de l'extension
     integer imex     ! nombre d'extensions
     integer xnext    ! next available entry
     integer ex(mex)  ! extension block address
  end type indexblock
  !
  ! note that the "dummies" are needed to match the file format...
  type obsblock
     sequence
     integer code    ! code observation icode
     integer nbloc   ! number of blocks
     integer nword   ! number of words 
     integer dumm4   ! ih(4)
     integer adata   ! data address
     integer ldata   ! data length
     integer dumm7   ! ih(7)
     integer nsec    ! number of section
     integer xnum    ! observation number  ixnum
  end type obsblock
  !
  type (obsblock) :: ib      ! input block structure
  !
  integer(kind=4) :: ibloc   ! bloc de l'obs courante
  integer(kind=4) :: clun    ! lun de l'observation courante
  integer(kind=4) :: maxsec  ! nb. max de sections
  !
  integer(kind=4) :: ikbl                     ! offset actuel dans l'obs. (blocs)
  integer(kind=4) :: code0,code1,code2,code3  ! codes d'identification
  integer(kind=4) :: isec(msec)               ! numeros des sections
  integer(kind=4) :: ilen(msec)               ! longueurs des sections
  integer(kind=4) :: iadd(msec)               ! addresses des sections
  integer(kind=4) :: xlun,xbl                 ! unite log. et bloc contenu ds xbuf
  integer(kind=4) :: old_ixnext               ! last known data in input file.
  !
  integer, parameter :: lenbuf=128  !
  integer(kind=4) :: ibuf(lenbuf)   ! buffer de lecture/ecriture
  integer(kind=4) :: xbuf(lenbuf)   ! buffer de lec/ecr. d'index
  !
  logical :: modify         ! the current observation is open for modification
  logical :: share          ! the output file is shared
  logical :: from_old       ! reading from old data format (i.e. pdp11)
  real(kind=4) :: interval  ! wait time for new_data
  integer :: number_trys    !
  logical :: chase_nan      ! Deeply track NaN while reading
  !
  ! Type which describe the CLASS files
  type class_file
     character(len=256) :: spec   ! File name
     integer(kind=4)    :: nspec  ! Filename length
     integer(kind=4)    :: lun    ! Logical unit
     type(indexblock)   :: bx     ! Block index of the file
     integer(kind=4)    :: conve  ! Conversion code
     ! Routines for system-data-type <-> file-data-type conversions
#if defined(FORTRAN2003_PP) /* Fortran 2003 Procedure Pointers */
     procedure(), nopass, pointer :: i4 => null()  ! for integer*4 values
     procedure(), nopass, pointer :: r4 => null()  ! for real*4 values
     procedure(), nopass, pointer :: r8 => null()  ! for real*8 values
     procedure(), nopass, pointer :: cc => null()  ! for character*(*) values
#else
     integer(kind=address_length) :: i4 = 0
     integer(kind=address_length) :: r4 = 0
     integer(kind=address_length) :: r8 = 0
     integer(kind=address_length) :: cc = 0
#endif
  end type class_file
  !
  type(class_file), save :: filein,fileout
  !
end module class_common
