module class_index
  use class_types 
  !
  type (indx) :: ind     ! Header de l'index pour l'observation courante
  type (indx) :: smin    ! Valeurs minimales pour FIND
  type (indx) :: smax    ! Valeurs maximales pour FIND
  !
  type (optimize), target :: ix  ! Index du fichier d'entree
  type (optimize), target :: ox  ! Index du fichier de sortie
  type (optimize), target :: cx  ! Index de la selection courante
  type (sorted) :: ox_sort
  !
  type(toc_t) :: ix_toc  ! Table of content of input file
  type(toc_t) :: ox_toc  ! Table of content of output file
  type(toc_t) :: cx_toc  ! Table of content of current index
  !
  type (flag) :: flg     ! Flag for selecting data
  !
  ! Index numbers of observation in memory. If 0 : no obs in memory,
  ! if -1 : an obs in memory that  was not read in a file.
  ! knext : pointer to the next index entry to consider
  integer knext          ! Pointer to next observation
  integer nindex         ! Index size 
  logical use_fox        ! Temporary flag to switch back to "old" FOX
end module class_index
!
module class_data
  use class_types
  use class_index
  type (observation) :: R ! Current observation
  type (observation) :: T ! Previous observation 
  type (observation) :: P ! Plotted observation (from LOAD) 
  type (indx) :: t_index
  integer ckind           ! Kind of current observation
  !
  real(kind=data_length), allocatable, target :: rdatas(:), rdatai(:), rdatav(:), rdatax(:)
  real, allocatable, target :: rdatay(:), rdataw(:)
  real(kind=data_length), allocatable, target :: tdatas(:), tdatai(:), tdatav(:), tdatax(:)
  real, allocatable, target :: tdatay(:), tdataw(:)
  real, allocatable, target :: rdata1(:), tdata1(:)
  !
  real(kind=data_length), allocatable, target :: pdatas(:), pdatai(:), pdatav(:), pdatax(:)
  real, allocatable, target :: pdataw(:)
  real(kind=data_length), allocatable, target :: qdatas(:), qdatai(:), qdatav(:), qdatax(:)
  real, allocatable, target :: qdataw(:)
  real, allocatable, target :: pdata2(:,:), qdata2(:,:)
end module class_data
!
module class_setup
  use class_types
  type(setup) :: set
  type(title) :: titre
  real(kind=4), allocatable :: window_polygon(:,:)
end module class_setup
!
module class_virtual
  use gildas_def
  !------------------------------------------------------------------------
  ! Support variables for the virtual buffer (SET VIRTUAL).
  !------------------------------------------------------------------------
  ! 
  logical,                      allocatable :: got_header(:)    ! Is the header in memory?
  integer(kind=address_length), allocatable :: v_header(:)      ! Memory address of the header
  logical,                      allocatable :: got_data(:)      ! Is the data section in memory?
  integer(kind=address_length), allocatable :: v_data(:)        ! Memory address of the data section
  !
  ! Header memory buffer
  integer(kind=address_length) :: header_memory_address ! Memory address of the header buffer
  integer :: header_memory_length                       ! Memory length of the header buffer
  integer :: header_memory_offset                       ! Offset of the next available memory address in the header buffer
  integer, save :: header_length
  integer, save :: number_of_slots
  integer, save :: current_slot
  integer, allocatable, save :: header_number(:)
  !
  ! Data memory buffer
  integer(kind=address_length) :: data_memory_address   ! Memory address of the data buffer
  integer :: data_memory_length                         ! Memory length of the data buffer
  integer :: data_memory_offset                         ! Offset of the next available memory address in the data buffer
  integer, parameter :: n_regions=64  ! Number of regions
  integer, save :: current_region  ! Active region
  integer, save :: region_length   ! Region length
  integer, save :: smallest_length ! Smallest unit
  integer, save :: region_size     ! Max Number of observations per region
  integer, allocatable, save :: number_of_obs(:) ! Cur Number of obs per region
  integer, allocatable, save :: obs_numbers(:,:)
  !
  ! Support of GET bug corrected on 2008-11-06
  logical :: virtual_bug_20081106=.false.
  !
end module class_virtual
