module class_parameter
  integer, parameter :: data_length=8
  !
  integer, parameter ::  mx_sec=64       ! Maximum range of section number
  integer, parameter ::  user_sec=0      ! User sections
  integer, parameter ::  com_sec=-1      ! Comment
  integer, parameter ::  gen_sec=-2      ! General
  integer, parameter ::  pos_sec=-3      ! Position
  integer, parameter ::  spec_sec=-4     ! Spectroscopy
  integer, parameter ::  base_sec=-5     ! Baseline
  integer, parameter ::  orig_sec=-6     ! Origin
  integer, parameter ::  plot_sec=-7     ! Plot s
  integer, parameter ::  switch_sec=-8   ! Phase description
  integer, parameter ::  fsw_sec=-8      ! Alias for phase description
  integer, parameter ::  gaus_sec=-9     ! Gauss fit
  integer, parameter ::  cont_sec=-10    ! Drift Continuum section
  integer, parameter ::  beam_sec=-11    ! Beam Switch
  integer, parameter ::  shell_sec=-12   ! Shell fit
  integer, parameter ::  nh3_sec=-13     ! NH3 fit
  integer, parameter ::  abs_sec=-18     ! Absorption fit
  integer, parameter ::  cal_sec=-14     ! Calibration
  integer, parameter ::  point_sec=-15   ! Continuum fit results
  integer, parameter ::  skydip_sec=-16  ! Skydip description
  integer, parameter ::  xcoord_sec=-17  ! Irregularly sampled X coordinates
  integer, parameter ::  descr_sec=-30   ! Old OTF format description (kept for backward compatibility)
  !
  ! Windows and Masks
  integer, parameter :: mwind1=100
  integer, parameter :: mmask1=100
  !
  ! Parameters that can be modified
  integer :: m_ix           ! Max number of obs in input index
  integer :: m_ox           ! Max number of obs in output index
  integer :: m_cx           ! Max number of obs in current index
  integer :: mdata = (32)   ! Max number of channels
  !
end module class_parameter
!
module class_consistency
  !
  use class_parameter
  !
  type :: consistency_section
     logical :: check ! [ ] Check this section?
     logical :: done  ! [ ] Already checked?
     logical :: prob  ! [ ] Inconsistent?
     logical :: mess  ! [ ] Output message?
     integer :: num   ! [ ] Observation number of inconsistent spectrum
  end type consistency_section
  !
  type :: consistency
     logical :: check       ! [ ] Anything left to be done?
     logical :: prob        ! [ ] Is there a consistency problem?
     !
     type(consistency_section) :: gen ! [ ] Generalities
     type(consistency_section) :: sou ! [ ] Source name
     type(consistency_section) :: lin ! [ ] Line name
     type(consistency_section) :: pos ! [ ] Position information
     type(consistency_section) :: spe ! [ ] Spectroscopy information
     !
     real(kind=8)           :: ftole ! [ MHz]  Frequency tolerance
     real(kind=data_length) :: vtole ! [km/s] Velocity  tolerance
  end type consistency
  !
end module class_consistency
!
module class_resample
  !
  type :: resampling ! Definition of a regularly sampled X-axis
     integer          :: nchan ! Number of channels
     real(kind=8)     :: ref   ! Reference channel
     real(kind=8)     :: val   ! Value at reference channel
     real(kind=8)     :: inc   ! Increment
     character(len=1) :: unit  ! Unit (i.e. frequency or velocity)
     character(len=8) :: shape ! Convolution kernel kind
     real             :: width ! Convolution kernel width
  end type resampling
  !
end module class_resample
!
module class_user
  use gildas_def
  !
  type :: userhooks
    character(len=12) :: owner
    character(len=12) :: title
#if defined(FORTRAN2003_PP) /* Fortran 2003 Procedure Pointers */
    procedure(), nopass, pointer :: toclass => null()
    procedure(), nopass, pointer :: dump    => null()
    procedure(), nopass, pointer :: setvar  => null()
#else
    integer(kind=address_length) :: toclass = 0
    integer(kind=address_length) :: dump    = 0
    integer(kind=address_length) :: setvar  = 0
#endif
  end type
  !
  type(userhooks), save :: userhook
  !
  integer(kind=4) :: usub  ! The User Subsection number the hooks are defined for
  logical :: setvar_user  ! SET VARIABLE USER is ON or OFF?
  !
end module class_user
!
module class_types
  use gildas_def
  use class_parameter
  use class_consistency
  !
  ! Section 0
  ! USER: User defined sections
  type user_sub  ! Type for each user subsection
     sequence
     character(len=12)        :: owner    ! Owner of the subsection
     character(len=12)        :: title    ! Title of the subsection
     integer(kind=4)          :: version  ! Version of the subsection
     integer(kind=4)          :: ndata    ! Length of the data(:) array
     integer(kind=4), pointer :: data(:)  ! Place holder for information
  end type
  !
  type user  ! Type for all user subsections
     sequence
     integer(kind=4)          :: n       ! Number of user subsections
     type (user_sub), pointer :: sub(:)  ! Array of subsections
  end type
  !
  ! Section -2 /CRPAR/
  ! GENERAL: General parameters, always present.
  integer, parameter :: mgen=9
  integer, parameter :: mgen2=12
  type general
     sequence
     integer(kind=4) :: num     ! [         ] Observation number
     integer(kind=4) :: ver     ! [         ] Version number
     integer(kind=4) :: teles(3)! [         ] Telescope name
     integer(kind=4) :: dobs    ! [MJD-60549] Date of observation
     integer(kind=4) :: dred    ! [MJD-60549] Date of reduction
     integer(kind=4) :: typec   ! [     code] Type of coordinates
     integer(kind=4) :: kind    ! [     code] Type of data
     integer(kind=4) :: qual    ! [     code] Quality of data
     integer(kind=4) :: scan    ! [         ] Scan number
     integer(kind=4) :: subscan ! [         ] Subscan number
     !
     real(kind=8)    :: ut      ! [  rad] UT of observation
     real(kind=8)    :: st      ! [  rad] LST of observation
     real(kind=4)    :: az      ! [  rad] Azimuth
     real(kind=4)    :: el      ! [  rad] Elevation
     real(kind=4)    :: tau     ! [neper] Opacity
     real(kind=4)    :: tsys    ! [    K] System temperature
     real(kind=4)    :: time    ! [    s] Integration time
     integer(kind=4) :: xunit   ! [ code] X unit (if xcoord_sec is present)
     ! NOT in data ---
     character(len=12) :: cdobs ! date observation
     character(len=12) :: cdred ! date reduction
  end type general
  !
  ! Section -3
  ! POSITION: Position information.
  integer, parameter :: mpos=17  ! Padding and alignment must not be included
  type position
     sequence
     integer(kind=4) :: sourc(3) ! [   ] Source name
     real(kind=4)    :: epoch    ! [   ] Epoch of coordinates
     real(kind=8)    :: lam      ! [rad] Lambda
     real(kind=8)    :: bet      ! [rad] Beta
     real(kind=4)    :: lamof    ! [rad] Offset in Lambda
     real(kind=4)    :: betof    ! [rad] Offset in Beta
     integer(kind=4) :: proj     ! [rad] Projection system
     integer(kind=4) :: align_1  ! [   ] Alignment padding
     real(kind=8)    :: sl0p     ! [rad] lambda of descriptive system
     real(kind=8)    :: sb0p     ! [rad] beta   of descriptive system
     real(kind=8)    :: sk0p     ! [rad] angle  of descriptive system
  end type position
  !
  ! Section -4
  ! SPECTRO: Spectroscopic information (for spectra).
  integer, parameter :: mspec=17      ! Padding and alignment must not be included
  integer, parameter :: mspec_old=18  ! Old spectro section (with SKYFR and
                        ! VTELES) was 18 words long. Need to read those 18
                        ! words for some old data (in particular CSO)
  type spectro
     sequence
     integer(kind=4) :: align   ! [    ] Alignment padding
     integer(kind=4) :: line(3) ! [    ] Line name
     real(kind=8)    :: restf   ! [ MHz] Rest frequency
     integer(kind=4) :: nchan   ! [    ] Number of channels
     real(kind=4)    :: rchan   ! [    ] Reference channels
     real(kind=4)    :: fres    ! [ MHz] Frequency resolution
     real(kind=4)    :: foff    ! [ MHz] Frequency offset
     real(kind=4)    :: vres    ! [km/s] Velocity resolution
     real(kind=4)    :: voff    ! [km/s] Velocity at reference channel
     real(kind=4)    :: bad     ! [    ] Blanking value
     integer(kind=4) :: align_1 ! [    ] Alignment padding
     real(kind=8)    :: image   ! [ MHz] Image frequency
     integer(kind=4) :: align_2 ! [    ] Alignment padding
     integer(kind=4) :: vtype   ! [code] Type of velocity
     real(kind=8)    :: doppler ! [    ] Doppler factor = -V/c (CLASS convention)
     ! real(kind=8)    :: skyfr   ! Sky frequency
     ! real(kind=4)    :: vteles  ! Velocity of telescope relative to reference
     ! integer(kind=4) :: pad
  end type spectro
  !
  ! Section -5
  ! BASE: Baseline information (for spectra of drifts).
  integer, parameter :: mwind=100
  integer, parameter :: mbase=8+2*mwind
  type base
     sequence
     integer(kind=4) :: deg         ! [         ] Degree of last baseline
     real(kind=4)    :: sigfi       ! [Int. unit] Sigma
     real(kind=4)    :: aire        ! [Int. unit] Area under windows
     integer(kind=4) :: nwind       ! [         ] Number of line windows
     real(kind=4)    :: w1(mwind)   ! [     km/s] Lower limits of windows
     real(kind=4)    :: w2(mwind)   ! [     km/s] Upper limits of windows
     real(kind=4)    :: sinus(3)    ! [         ] Sinus baseline results
     real(kind=4)    :: chisq       ! [        ?] Chi square
  end type base
  !
  ! Section -6
  ! HISTORY: Scan numbers of initial observations.
  integer, parameter :: mseq=100
  integer, parameter :: morig=2*mseq+1
  type history
     sequence
     integer(kind=4) :: nseq        ! Number of sequences
     integer(kind=4) :: start(mseq) ! Start can number of seq.
     integer(kind=4) :: end(mseq)   ! End scan number of seq.
     integer(kind=4) :: pad         ! Alignment padding
  end type history
  !
  ! Section -7
  ! PLOT: Default plotting limits.
  integer, parameter :: mplot=4
  type plot
     sequence
     real(kind=4)    :: amin    ! [Int. unit] Min Y value plotted
     real(kind=4)    :: amax    ! [Int. unit] Max Y value plotted
     real(kind=4)    :: vmin    ! [     km/s] Min X value plotted
     real(kind=4)    :: vmax    ! [     km/s] Max X value plotted
  end type plot
  !
  ! Section -8
  ! SWITCH: Switching information (for spectra).
  integer, parameter :: mxphas=8
  integer, parameter :: mfsw=2+6*mxphas
  type fswitch
     sequence
     integer(kind=4) :: align           ! [    ] Padding
     integer(kind=4) :: nphas           ! [    ] Number of phases
     real(kind=8)    :: decal(mxphas)   ! [ MHz] Frequency offsets
     real(kind=4)    :: duree(mxphas)   ! [   s] Time per phase
     real(kind=4)    :: poids(mxphas)   ! [    ] Weight of each phase
     integer(kind=4) :: swmod           ! [code] Switching mode (frequency, position...)
     real(kind=4)    :: ldecal(mxphas)  ! [ rad] Lambda offsets
     real(kind=4)    :: bdecal(mxphas)  ! [ rad] Beta offsets of each phase
     integer(kind=4) :: pad
  end type fswitch
  !
  ! Section -14
  ! CALIBRATION: Calibration parameters.
  integer, parameter :: mcalib=25
  type calib
     sequence
     integer(kind=4) :: align    ! [     ] Padding
     real(kind=4)    :: beeff    ! [     ] Beam efficiency
     real(kind=4)    :: foeff    ! [     ] Forward efficiency
     real(kind=4)    :: gaini    ! [     ] Image/Signal gain ratio
     real(kind=4)    :: h2omm    ! [   mm] Water vapor content
     real(kind=4)    :: pamb     ! [  hPa] Ambient pressure
     real(kind=4)    :: tamb     ! [    K] Ambient temperature
     real(kind=4)    :: tatms    ! [    K] Atmosphere temp. in signal band
     real(kind=4)    :: tchop    ! [    K] Chopper temperature
     real(kind=4)    :: tcold    ! [    K] Cold load temperature
     real(kind=4)    :: taus     ! [neper] Opacity in signal band
     real(kind=4)    :: taui     ! [neper] Opacity in image band
     real(kind=4)    :: tatmi    ! [    K] Atmosphere temp. in image band
     real(kind=4)    :: trec     ! [    K] Receiver temperature
     integer(kind=4) :: cmode    ! [ code] Calibration mode
     real(kind=4)    :: atfac    ! [     ] Applied calibration factor
     real(kind=4)    :: alti     ! [    m] Site elevation
     real(kind=4)    :: count(3) ! [count] Power of Atm., Chopp., Cold
     real(kind=4)    :: lcalof   ! [  rad] Longitude offset for sky measurement
     real(kind=4)    :: bcalof   ! [  rad] Latitude offset for sky measurement
     real(kind=8)    :: geolong  ! [  rad] Geographic longitude of observatory
     real(kind=8)    :: geolat   ! [  rad] Geographic latitude of observatory
  end type calib
  !
  ! Section -16
  ! SKYDIP: For Skydips observations. No associated data.
  integer, parameter :: msky=10
  integer, parameter :: mskydip=10+4*msky
  type skydip
     sequence
     integer(kind=4) :: align       ! [   ] Padding
     integer(kind=4) :: line(3)     ! [   ] Line name
     real(kind=8)    :: restf       ! [MHz] Rest frequency
     real(kind=8)    :: image       ! [MHz] Image frequency
     integer(kind=4) :: nsky        ! [   ] Number of points on sky
     integer(kind=4) :: nchop       ! [   ]   -   -    -    - chopper
     integer(kind=4) :: ncold       ! [   ]   -   -    -    - cold load
     real(kind=4)    :: elev(msky)  ! [rad] Elevations
     real(kind=4)    :: emiss(msky) ! [  ?] Power on sky
     real(kind=4)    :: chopp(msky) ! [  ?] Power on chopper
     real(kind=4)    :: cold(msky)  ! [  ?] Power on cold load
     integer(kind=4) :: pad         ! [   ] Padding
  end type skydip
  !
  ! Section -9  /CRGAUS/
  ! GAUSS: Gauss fit results (for spectra or drifts).
  integer, parameter :: mxgaus=5
  integer, parameter :: mgausfit=3*mxgaus
  integer, parameter :: mfit=3+2*mgausfit
  type gauss
     sequence
     integer(kind=4) :: nline           ! [            ] Number of components
     real(kind=4)    :: sigba           ! [   Int. unit] Sigma on base
     real(kind=4)    :: sigra           ! [   Int. unit] Sigma on line
     real(kind=4)    :: nfit(mgausfit)  ! [area,v0,fwhm] Fit results
     real(kind=4)    :: nerr(mgausfit)  ! [area,v0,fwhm] Errors
     real(kind=4)    :: pad             ! [            ] Alignement padding
  end type gauss
  !
  ! Section -12   /CRSHEL/
  ! SHELL: "Stellar shell" profile fit results (for spectra).
  integer, parameter :: mshellfit=20
  integer, parameter :: mshell=3+2*mshellfit
  type shell
     sequence
     integer(kind=4) :: nline            ! [] Number of components
     real(kind=4)    :: sigba            ! [] Sigma on base
     real(kind=4)    :: sigra            ! [] Sigma on line
     real(kind=4)    :: nfit(mshellfit)  ! [] Fit results
     real(kind=4)    :: nerr(mshellfit)  ! [] Errors
     real(kind=4)    :: pad              ! [] Alignement padding
  end type shell
  !
  ! Section -13 /CRNH3/
  ! HFS: "Hyperfine Structure" profile fit results (e.g. NH3, HCN, for spectra).
  integer, parameter :: mnh3fit=12
  integer, parameter :: mnh3=3+2*mnh3fit
  type hfs
     sequence
     integer(kind=4) :: nline          ! Number of components
     real(kind=4)    :: sigba          ! Sigma on base
     real(kind=4)    :: sigra          ! Sigma on line
     real(kind=4)    :: nfit(mnh3fit)  ! Fit results
     real(kind=4)    :: nerr(mnh3fit)  ! Errors
     real(kind=4)    :: pad            ! Alignement
  end type hfs
  !
  ! Section -18 /CRABS/
  ! ABS: "Hyperfine Structure" absorption profile fit results (e.g. NH3, HCN, for spectra).
  integer, parameter :: mabsfit=16
  integer, parameter :: mabs=3+2*mabsfit
  type absorption
     sequence
     integer(kind=4) :: nline          ! Number of components
     real(kind=4)    :: sigba          ! Sigma on base
     real(kind=4)    :: sigra          ! Sigma on line
     real(kind=4)    :: nfit(mabsfit)  ! Fit results
     real(kind=4)    :: nerr(mabsfit)  ! Errors
     real(kind=4)    :: pad            ! Alignement
  end type absorption
  !
  ! Section -10 /CRCONT/
  ! DRIFT: Continuum drift description (for drifts).
  integer, parameter :: mcont=16
  type drift
     sequence
     real(kind=8)    :: freq    ! [MHz] Rest frequency
     real(kind=4)    :: width   ! [MHz] Bandwidth
     integer(kind=4) :: npoin   ! [   ] Number of data points
     real(kind=4)    :: rpoin   ! [   ] Reference point
     real(kind=4)    :: tref    ! [  ?] Time at reference
     real(kind=4)    :: aref    ! [rad] Angular offset at ref.
     real(kind=4)    :: apos    ! [rad] Position angle of drift
     real(kind=4)    :: tres    ! [  ?] Time resolution
     real(kind=4)    :: ares    ! [rad] Angular resolution
     real(kind=4)    :: bad     ! [   ] Blanking value
     integer(kind=4) :: ctype   ! [code] Type of offsets
     real(kind=8)    :: cimag   ! [MHz] Image frequency
     real(kind=4)    :: colla   ! [] Collimation error Az
     real(kind=4)    :: colle   ! [] Collimation error El
  end type drift
  !
  ! Section -11
  ! BEAM: Beam-switching parameters (for spectra or drifts).
  integer, parameter :: mbeam=5
  type beam
     sequence
     real(kind=4)    :: cazim   ! [ rad] Azimuth of observation
     real(kind=4)    :: celev   ! [ rad] Elevation of observation
     real(kind=4)    :: space   ! [ rad] Beam spacing
     real(kind=4)    :: bpos    ! [ rad] Position angle of beams
     integer(kind=4) :: btype   ! [code] System for angle
     integer(kind=4) :: pad     ! [    ] Alignment padding
  end type beam
  !
  ! Section -15 /CRPOINT/
  ! CONTINUUM: Double gaussian and baseline fit results (for drifts).
  integer, parameter :: mfcontfit=8
  integer, parameter :: mfcont=3+2*mfcontfit
  type pointing
     sequence
     integer(kind=4) :: nline            ! Number of components
     real(kind=4)    :: sigba            ! Sigma on base
     real(kind=4)    :: sigra            ! Sigma on line
     real(kind=4)    :: nfit(mfcontfit)  ! Fit results
     real(kind=4)    :: nerr(mfcontfit)  ! Errors
     integer(kind=4) :: pad              ! Alignment padding
  end type pointing
  !
  ! Section -30
  ! Data Section Descriptor.
  ! (Old OTF data format, kept for backward compatibility).
  integer, parameter :: mdescr=4
  type descriptor
     sequence
     integer(kind=4) :: ndump   ! Number of records
     integer(kind=4) :: ldpar   ! Length of data header (longwords)
     integer(kind=4) :: ldatl   ! length of line data      (")
     integer(kind=4) :: ldump   ! length of record         (")
     integer(kind=4) :: rec     ! Current Record Number
     integer(kind=4) :: pad     ! Alignment padding
  end type descriptor
  !
  integer, parameter :: mcomm=256
  type comment
     sequence
     integer(kind=4) :: ltext   ! Length of comment
     integer(kind=4) :: pad     ! Alignment padding
     character ctext*1024       ! Comment string
  end type comment
  !
  ! Observation Header
  type header
     sequence
     integer(kind=4)  :: start(2)
     type(general)    :: gen
     type(position)   :: pos
     type(spectro)    :: spe
     type(base)       :: bas
     type(history)    :: his
     type(plot)       :: plo
     type(fswitch)    :: fsw
     type(calib)      :: cal
     type(skydip)     :: sky
     type(gauss)      :: gau
     type(shell)      :: she
     type(hfs)        :: hfs
     type(absorption) :: abs
     type(drift)      :: dri
     type(beam)       :: bea
     type(pointing)   :: poi
     type(descriptor) :: des
     type(comment)    :: com
     !
     logical          :: presec(-mx_sec:0) ! Which section are presents
     integer          :: xnum              ! Origin of spectrum (data/model/fit/absent)
     integer(kind=4)  :: ends(2)
  end type header
  !
  ! Data additional parameters
  ! (Old OTF data format, kept for backward compatibility)
  type dapotf
     integer :: irec   ! Record number
     real :: az        ! Azimut
     real :: el        ! Elevation
     real :: st        ! LST
     real :: ut        ! UT
     real :: lamof     ! Offset in Lambda
     real :: betof     ! Offset in Beta
     integer :: feed   ! Mutli-beam feed number
  end type dapotf
  !
  type observation
     type(header) :: head
     type(comment) :: note
     integer cimin, cimax    ! First and last channel displayed
     integer :: cnchan       ! Number of data points
     real :: cbad            ! Bad value
     real :: rx_off, ry_off  ! Current offsets
     logical :: is_otf       ! Is an old OTF scan?
     ! Abscissa
     real(kind=data_length), pointer :: datax (:) ! Current X abscissa
     real(kind=data_length), pointer :: datas (:) ! Signal frequencies
     real(kind=data_length), pointer :: datai (:) ! Image frequencies
     real(kind=data_length), pointer :: datav (:) ! Velocities
     ! Values
     real(kind=4), pointer :: data1 (:)   ! 1D Values
     real(kind=4), pointer :: dataw (:)   ! Weights
     real(kind=4), pointer :: data2 (:,:) ! 2D Values
     real(kind=4), pointer :: spectre (:) ! Y Values
     type(dapotf), pointer :: dap(:)      ! OTF daps
     ! User section
     type(user) :: user
  end type observation
  !
  type indx
     sequence
     integer(kind=4) :: bloc    ! block red from index
     integer(kind=4) :: num     ! number red
     integer(kind=4) :: ver     ! version red from index
     character(len=12) :: csour ! source red from index
     character(len=12) :: cline ! line red from index
     character(len=12) :: ctele ! telescope red from index
     integer(kind=4) :: dobs    ! date obs. red from index
     integer(kind=4) :: dred    ! date red. red from index
     real(kind=4)    :: off1    ! red offset 1
     real(kind=4)    :: off2    ! red offset 2
     integer(kind=4) :: type    ! type of red offsets
     integer(kind=4) :: kind    ! type of observation   KIND = 0 Spectre, 1 Continuum
     integer(kind=4) :: qual    ! Quality red from index
     integer(kind=4) :: scan    ! Scan number red from index
     real(kind=4)    :: posa    ! Position angle
     integer(kind=4) :: subscan ! Subscan number
     integer(kind=4) :: pad(10) ! Pad to 32 integers
  end type indx
  !
  type ranges_t
     logical :: done                          ! Already done or not
     integer :: scan1                         ! First scan number
     integer :: scan2                         ! Last  scan number
     integer :: dobs1                         ! Minimum OBS date
     integer :: dobs2                         ! Maximum OBS date
     integer :: dred1                         ! Minimum RED date
     integer :: dred2                         ! Maximum RED date
     real(4) :: lam1                          ! Minimum Lambda offset
     real(4) :: lam2                          ! Maximum Lambda offset
     real(4) :: bet1                          ! Minimum Beta   offset
     real(4) :: bet2                          ! Maximum Beta   offset
  end type ranges_t
  !
  type toc_t
     integer :: nsetup                        ! Number of setup (Source+Line+Backend)
     integer :: nsour                         ! Number of sources
     integer :: nline                         ! Number of lines
     integer :: ntele                         ! Number of telescopes
     integer :: ndobs                         ! Number of observation dates
     integer, pointer :: msour(:)             ! Number of obs in sour  section
     integer, pointer :: mline(:)             ! Number of obs in line  section
     integer, pointer :: mtele(:)             ! Number of obs in tele  section
     integer, pointer :: mdobs(:)             ! Number of obs in dobs  section
     integer, pointer :: msetup(:)            ! Number of obs in setup section
     character(len=12), pointer :: sour(:)    ! Source names
     character(len=12), pointer :: line(:)    ! Line names
     character(len=12), pointer :: tele(:)    ! Telescope names
     character(len=12), pointer :: dobs(:)    ! Observation dates
     character(len=12), pointer :: setup(:,:) ! Setup names
  end type toc_t
  !
  type optimize
     logical :: has_old_fmt                   ! =.true. if (at least one) subscan=0
     integer :: time                          ! Time when index is filled
     integer :: next                          ! Number of observations in index + 1
     type(consistency) :: cons                ! Consistency checks, tolerances and results
     type(ranges_t)    :: ranges              ! Ranges of the INDEX
     integer, pointer :: ind(:)               ! Index values (C)
     integer, pointer :: num(:)               ! Observation numbers (I/O/C)
     integer, pointer :: bloc(:)              ! Bloc numbers (I/O/C)
     integer, pointer :: ver(:)               ! Version numbers (I/O/C)
     integer, pointer :: kind(:)              ! Kinds (I)
     integer, pointer :: qual(:)              ! Qualities (I)
     integer, pointer :: scan(:)              ! Scan numbers (I)
     integer, pointer :: dobs(:)              ! Observation dates (I)
     real(4), pointer :: off1(:)              ! Lambda Offset (R)
     real(4), pointer :: off2(:)              ! Beta Offset (R)
     integer, pointer :: subscan(:)           ! Subscan number  (I)
     character(len=12), pointer :: csour(:)   ! Source name
     character(len=12), pointer :: cline(:)   ! Line name
     character(len=12), pointer :: ctele(:)   ! Backend name
  end type optimize
  !
  ! Sorted index. An implementation with different ordering may be faster
  type sorted
     integer :: nsort
     integer, pointer :: number(:)
     integer, pointer :: entry(:)
  end type sorted
  !
  type flag
     logical         :: last     ! only last version
     logical         :: ind      ! by entry number
     logical         :: num      ! by observation number
     logical         :: ver      ! by version
     logical         :: sourc    ! by source name
     integer(kind=4) :: isourc   ! (first wildcard position)
     logical         :: line     ! by line name
     integer(kind=4) :: iline    ! (first wildcard position)
     logical         :: teles    ! by backend
     integer(kind=4) :: iteles   ! (first wildcard position)
     logical         :: dobs     ! by observation date
     logical         :: dred     ! by reduction date
     logical         :: off1     ! by offset 1
     logical         :: off2     ! by offset 2
     logical         :: scan     ! by scan number
     logical         :: subscan  ! by subscan number
     logical         :: posa     ! by position angle
     ! Other criteria which do not rely on the index:
     logical         :: sect     ! by section present
     integer(kind=4) :: sectval  ! section identifier
     logical         :: freq     ! by frequency
     real(kind=8)    :: freqmin  ! frequency range: minimal value
     real(kind=8)    :: freqmax  ! frequency range: maximal value
     logical         :: freqsig  ! Signal or image frequency selection?
  end type flag
  !
  type setup
     real(kind=8)   :: fangle           ! Angle conversion factor
     real(kind=8)   :: freqmin,freqmax  ! Frequency range for command FIND
     logical        :: freqsig          ! Signal or image frequency selection?
     real(kind=4)   :: offs1, offs2     ! Offset Min Max (E-W)
     real(kind=4)   :: offl1, offl2     ! Offset Min Max (N-S)
     real(kind=4)   :: tole             ! Tolerance on position checks
     real(kind=4)   :: beamt, gaint     ! Tolerance on BeamEff and GainImage
     real(kind=4)   :: wind1(mwind1)    ! Windows definition
     real(kind=4)   :: wind2(mwind1)    ! Windows definition
     real(kind=4)   :: mask1(mmask1)    ! Masks definition
     real(kind=4)   :: mask2(mmask1)    ! Masks definition
     real(kind=4)   :: epoch            ! Default epoch
     real(kind=4)   :: posa1, posa2     ! Position angle
     integer(kind=4) :: nume1, nume2    ! Observation Numbers
     integer(kind=4) :: scan1, scan2    ! Scan Numbers
     integer(kind=4) :: sub1,sub2       ! Subscan numbers
     integer(kind=4) :: base            ! Baseline degree (-1 = Sinus)
     integer(kind=4) :: nwind           ! Window number
     integer(kind=4) :: nmask           ! Mask number
     integer(kind=4) :: kind            ! 0 Line, 1 Continuum
     integer(kind=4) :: obse1, obse2    ! Observing dates
     integer(kind=4) :: redu1, redu2    ! Reduction dates
     integer(kind=4) :: coord           ! Coordinate Type
     integer(kind=4) :: veloc           ! Velocity Type
     integer(kind=4) :: lbase           ! Last Baseline degree
     integer(kind=4) :: qual1, qual2    ! Quality of data
     integer(kind=4) :: slev, flev      ! Output message priorities (screen and file)
     integer(kind=4) :: drop(2)         ! Drop edge channels at reading
     logical :: write_r8                ! Write Irregularly sampled data as Real*8
     logical :: match                   ! Position checking
     logical :: mapcl                   ! clear in MAP
     logical :: fupda                   ! FIND UPDATE mode
     logical :: do_sort                 ! SORT index
     logical :: do_scan                 ! SCAN Mode for PLOT, BASE, FOLD, ...
     logical :: win_pol                 ! SET WINDOW /POLYGON
     logical :: origin                  ! List the history ?
     logical :: verbose                 ! Verbose printout
     logical :: fft                     ! Fourier space (True) or Direct space
     logical :: virtual                 ! Use or not VM "optimization" ?
     logical :: virtual_check           ! Check the bug corrected on 06-nov-2008?
     character(len=12) :: sort_name     ! Sort key name
     !
     character(len=12) :: line      ! Line Name
     character(len=12) :: sourc     ! Source Name
     character(len=12) :: teles     ! Telescope Name
     character(len=12) :: method    ! Fit Method
     character(len=32) :: defext    ! Default Extension
     character(len=3) :: angle      ! Angle unit
     character(len=1) :: heade      ! Header type
     character(len=1) :: modex      ! Plot Mode
     character(len=1) :: modey      ! Plot Mode
     character(len=1) :: modez      ! Plot Mode
     character(len=1) :: plot       ! Plot Type (normal, histogram)
     character(len=1) :: pl_lev     ! Plot Level (scan, index)
     character(len=1) :: unitx(2)   ! Abscissae axis unit (lower and upper)
     character(len=1) :: weigh      ! Summation Weight
     character(len=1) :: bad        ! Bad checking
     character(len=1) :: action     ! Index or Observation
     character(len=1) :: box        ! Box orientation (L or P)
     character(len=1) :: align
     character(len=1) :: alig2
     character(len=32) :: winpol_name(mwind1)
  end type setup
  !
  type title
     logical :: position
     logical :: quality
     logical :: spectral
     logical :: calibration
     logical :: atmosphere
     logical :: origin
     logical :: continuum
  end type title
end module class_types
!
! Others..
!
module gauss_parameter
  use class_parameter
  integer, parameter :: mxline=5
  !
  ! Initial values and Dependency codes
  real(kind=4) :: spar(4*mxline),deltav   ! 3 for Gauss, 4 for HFS - NH3,
  integer :: kt0,kt(5)            ! 1,kt2,kt3,kt4,kt5
  integer :: kv0,kv(5)            ! 1,kv2,kv3,kv4,kv5
  integer :: kd0,kd(5)            ! 1,kd2,kd3,kd4,kd5
  integer :: ko0,ko(5)            ! 1,ko2,ko3,ko4,ko5
  integer :: kh0,kh(5)            !
  integer :: nline
  !
  ! Weights for minimization
  integer :: mxcan
  integer, allocatable :: wfit(:)
  !
  ! Common containing results and errors
  integer :: ngline               ! Number of lines
  real(kind=4) :: par(4*mxline)   ! Parameters
  real(kind=4) :: err(4*mxline)   ! Errors
  real(kind=4) :: sigbas          ! rms on baseline
  real(kind=4) :: sigrai          ! rms on line
end module gauss_parameter
!
!
!
module sinus_parameter
  use class_parameter
  integer, parameter :: mxline=5
  !
  ! Initial values and Dependency codes
  real(4), dimension(mxline) :: spar
  real(4)                    :: deltav
  ! Weights for minimization
  integer              :: mxcan
  integer, allocatable :: wfit(:)
  ! Common containing results and errors
  real(4), dimension(mxline) :: par     ! Parameters
  real(4), dimension(mxline) :: err     ! Errors
  real(4)                    :: sigbas  ! rms on baseline
  real(4)                    :: sigrai  ! rms on area under line
end module sinus_parameter
!
module class_buffer
  use class_types
  !
  integer(kind=4), allocatable :: iwork(:)  ! Buffer for standard sections
  !
  integer(kind=4), save :: jlen=0
  integer(kind=4), allocatable :: jwork(:)  ! Buffer for X coordinates section
  !
  integer(kind=4), parameter :: ulen=512  ! Maximum User Section size
  integer(kind=4) :: uwork(ulen)          ! Buffer for User Section
  integer(kind=4) :: unext                ! Next element to write in uwork
  !
end module class_buffer
